package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.*;

import java.io.*;
import javax.microedition.lcdui.*;

/**
 * Customized alert screen.
 * @author Adam Netocny
 */
public class AlertScreen
        extends Form
        implements  CommandListener,
                    Displayable
{

    /**
     * Error icon.
     */
    private Image error_image = null;
    
    /**
     * Warning icon.
     */
    private Image warning_image = null;
    
    /**
     * Info icon.
     */
    private Image info_image = null;

    /**
     * Exit command.
     */
    private final Command exit = new Command("Exit", Command.EXIT, 1);

    /**
     * Back command.
     */
    private final Command back = new Command("Back", Command.BACK, 1);

    /**
     * Constructor. Creates all necessary items on the screen.
     * @param alertString String to be shown.
     * @param type Alert type.
     *
     * @see AlertType
     */
    public AlertScreen(String alertString, AlertType type)
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME);

        loadImages();
        
        processTypeInfo(type);

        append(alertString);
        
        setCommandListener(this);
    }

    /**
     * Load images.
     */
    private void loadImages()
    {
        try
        {
            error_image = Image.createImage("/resources/error.png");
        }
        catch (IOException ex)
        { }

        try
        {
            info_image = Image.createImage("/resources/info.png");
        }
        catch (IOException ex)
        { }

        try
        {
            warning_image = Image.createImage("/resources/warning.png");
        }
        catch (IOException ex)
        { }
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == exit)
        {
            synchronized(TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.exitMIDlet();
            }
        }
        else if (c == back)
        {
            synchronized(TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.screenBack();
            }
        }
    }

    /**
     * Processes <tt>AlertType</tt> information.
     * @param type AlertType.
     */
    private void processTypeInfo(AlertType type) {
        if (type == AlertType.ALARM || type == AlertType.ERROR)
        {
            if(error_image != null)
            {
                append(error_image);
            }
            addCommand(exit);
        }
        else if(type == AlertType.WARNING)
        {
            if(warning_image != null)
            {
                append(warning_image);
            }
            addCommand(back);
        }
        else
        {
            if(info_image != null)
            {
                append(info_image);
            }
            addCommand(back);
        }
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        return this;
    }
}
