package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.*;
import java.io.IOException;

import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

/**
 * File browser screen.
 * @author Adam Netocny
 */
public class FileBrowser
        extends List
        implements Displayable,
                   CommandListener
{

    /**
     * Currently shown directory.
     */
    private String currDirName;

    /**
     * View command.
     */
    private final Command view = new Command("View", Command.ITEM, 0);

    /**
     * Check command.
     */
    private final Command check = new Command("Check", Command.ITEM, 0);

    /**
     * Start selecting command.
     */
    private final Command selectOn = new Command(
            "Select more...", Command.ITEM, 1);

    /**
     * Stop selecting command.
     */
    private final Command selectOff = new Command(
            "Cancel", Command.ITEM, 1);

    /**
     * Add to tar command.
     */
    private final Command tar = new Command("Create TAR...", Command.ITEM, 2);

    /**
     * Exit command.
     */
    private final Command exit = new Command("Exit", Command.EXIT, 3);

    /**
     * File image.
     */
    private Image file_image = null;

    /**
     * Folder image.
     */
    private Image folder_image = null;

    /**
     * Root folder image.
     */
    private Image root_image = null;

    /**
     * Selected file image.
     */
    private Image selected_file_image = null;

    /**
     * Selected folder image.
     */
    private Image selected_folder_image = null;

    /**
     * Selected root image.
     */
    private Image selected_root_image = null;

    /**
     * Back image.
     */
    private Image back_image = null;

    /**
     * Up dir string.
     */
    private final static String UP_DIRECTORY = "..";

    /**
     * Root string.
     */
    private final static String MEGA_ROOT = "/";

    /**
     * Separator string.
     */
    private final static String SEP_STR = "/";

    /**
     * Separator char.
     */
    private final static char SEP = '/';

    /**
     * Selected items.
     */
    private final Vector selected = new Vector();

    /**
     * If true selection mode is turned on.
     */
    private boolean isSelecting = false;

    /**
     * Constructor. Adds all necessary items.
     */
    public FileBrowser()
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME
              + " - Filesystem roots", IMPLICIT);

        currDirName = MEGA_ROOT;

        addCommand(selectOn);
        setSelectCommand(view);
        addCommand(exit);
        setCommandListener(this);

        try
        {
            folder_image = Image.createImage("/resources/folder.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            file_image = Image.createImage("/resources/file.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            root_image = Image.createImage("/resources/root.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            selected_folder_image = Image.createImage("/resources/folderSel.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            selected_file_image = Image.createImage("/resources/fileSel.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            selected_root_image = Image.createImage("/resources/rootSel.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            back_image = Image.createImage("/resources/back.png");
        }
        catch (IOException ex)
        {
        }
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        if (System.getProperty(
                "microedition.io.file.FileConnection.version") != null)
        {
            try
            {
                if (showCurrDir())
                {
                    return this;
                }
                else
                {
                    return getAlert();
                }
            }
            catch (SecurityException e)
            {
                return getAlert();
            }
            catch (Throwable e)
            {
                return getAlert();
            }
        }
        else
        {
            return getAlert();
        }
    }

    /**
     * Gets alert screen.
     * @return Alert screen.
     */
    private Form getAlert()
    {
        return new AlertScreen("FileConnection API not available!",
                AlertType.ERROR);
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == exit)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.exitMIDlet();
            }
        }
        else if (c == view)
        {
            List curr = (List) d;
            final String currFile = curr.getString(curr.getSelectedIndex());
            new Thread(new Runnable()
            {

                public void run()
                {
                    if (currFile.endsWith(SEP_STR)
                        || currFile.equals(UP_DIRECTORY))
                    {
                        traverseDirectory(currFile);
                    }
                }
            }).start();

            if (!currFile.endsWith(SEP_STR) && !currFile.equals(UP_DIRECTORY))
            {
                synchronized (TarArchivatorMidlet.lockMe)
                {
                    if (!currFile.toLowerCase().endsWith("tar"))
                    {
                        try
                        {
                            TarArchivatorMidlet.me.switchDisplayable(new FileViewer(
                                    currDirName + currFile).getScreen());
                        }
                        catch (IOException ex)
                        {
                            ex.printStackTrace();
                        }
                    }
                    else
                    {
                        TarArchivatorMidlet.me.switchDisplayable(new UnTarScreen(
                                currDirName + currFile).getScreen());
                    }
                }

            }
        }
        else if (c == tar)
        {
            Vector x = selected;

            String[] files = new String[x.size()];
            for (int i = 0; i < x.size(); i++)
            {
                files[i] = (String) x.elementAt(i);
            }

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.tarFiles(files);
            }
        }
        else if (c == selectOn || c == selectOff)
        {
            if (!isSelecting)
            {
                isSelecting = true;
                setSelectCommand(check);
                removeCommand(view);
                removeCommand(selectOn);
                addCommand(selectOff);
                addCommand(tar);
            }
            else
            {
                isSelecting = false;
                setSelectCommand(view);
                removeCommand(check);
                removeCommand(selectOff);
                removeCommand(tar);
                addCommand(selectOn);

                for (int i = 0; i < selected.size(); i++)
                {
                    String currFile = (String) selected.elementAt(i);
                    currFile = currFile.substring(currDirName.length());
                    int index = getIndex(currFile);

                    if (index == -1)
                    {
                        continue;
                    }

                    if (getImage(index) == selected_file_image)
                    {
                        set(index, currFile, file_image);
                    }
                    else if (getImage(index)
                             == selected_folder_image)
                    {
                        set(index, currFile, folder_image);
                    }
                }

                selected.removeAllElements();
            }
        }
        else if (c == check)
        {
            List curr = (List) d;
            final String currFile = curr.getString(curr.getSelectedIndex());

            if (currFile.equals(UP_DIRECTORY)
                || getImage(curr.getSelectedIndex()) == root_image)
            {
                return;
            }

            if (selected.contains(currDirName + currFile))
            {
                selected.removeElement(currDirName + currFile);
                if (getImage(curr.getSelectedIndex()) == selected_file_image)
                {
                    set(curr.getSelectedIndex(), currFile, file_image);
                }
                else if (getImage(curr.getSelectedIndex())
                         == selected_folder_image)
                {
                    set(curr.getSelectedIndex(), currFile, folder_image);
                }
            }
            else
            {
                selected.addElement(currDirName + currFile);
                if (getImage(curr.getSelectedIndex()) == file_image)
                {
                    set(curr.getSelectedIndex(), currFile, selected_file_image);
                }
                else if (getImage(curr.getSelectedIndex()) == folder_image)
                {
                    set(curr.getSelectedIndex(), currFile, selected_folder_image);
                }
            }
        }
    }

    /**
     * Fills the list with file and folder list.
     * @return true if no exception occurs else false.
     */
    private boolean showCurrDir()
    {
        this.deleteAll();
        this.selected.removeAllElements();

        isSelecting = false;
        setSelectCommand(view);
        removeCommand(check);
        removeCommand(selectOff);
        removeCommand(selectOn);
        removeCommand(tar);
        addCommand(selectOn);

        Enumeration e;
        FileConnection currDir = null;
        boolean root = false;
        try
        {
            if (MEGA_ROOT.equals(currDirName))
            {
                removeCommand(selectOff);
                removeCommand(selectOn);
                removeCommand(tar);
                e = FileSystemRegistry.listRoots();
                root = true;
                this.setTitle(TarArchivatorMidlet.APP_DISPLAY_NAME
                              + " - Filesystem roots");
            }
            else
            {
                currDir = (FileConnection) Connector.open(
                        "file://localhost/" + currDirName);
                e = currDir.list();
                this.append(UP_DIRECTORY, back_image);
                this.setTitle(currDirName);
            }

            Vector folders = new Vector();
            Vector files = new Vector();
            while (e.hasMoreElements())
            {
                String fileName = (String) e.nextElement();
                if (root)
                {
                    folders.addElement(fileName);
                }
                else if (fileName.charAt(fileName.length() - 1) == SEP)
                {
                    folders.addElement(fileName);
                }
                else
                {
                    files.addElement(fileName);
                }
            }

            folders = sortStringVector(folders);
            files = sortStringVector(files);

            if (root)
            {
                appendVector(folders, root_image);
            }
            else
            {
                appendVector(folders, folder_image);
            }

            appendVector(files, file_image);

            if (currDir != null)
            {
                currDir.close();
            }
        }
        catch (Exception ioe)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.getDisplay().setCurrent(getAlert());
            }
            return false;
        }
        return true;
    }

    /**
     * Changes the current dir path.
     * @param fileName New file name.
     */
    public void traverseDirectory(String fileName)
    {
        if (currDirName.equals(MEGA_ROOT))
        {
            if (fileName.equals(UP_DIRECTORY))
            {
                // can not go up from MEGA_ROOT
                return;
            }
            currDirName = fileName;
        }
        else if (fileName.equals(UP_DIRECTORY))
        {
            int i = currDirName.lastIndexOf(SEP, currDirName.length() - 2);
            if (i != -1)
            {
                currDirName = currDirName.substring(0, i + 1);
            }
            else
            {
                currDirName = MEGA_ROOT;
            }
        }
        else
        {
            currDirName = currDirName + fileName;
        }
        showCurrDir();
    }

    /**
     * Sets the view to the top directory.
     */
    public void setTopDirView()
    {
        currDirName = MEGA_ROOT;
    }

    /**
     * Sorts  strings alphabetically.
     * @param en Vector of string.
     * @return Sorted vector of string.
     */
    private Vector sortStringVector(Vector en)
    {
        if (!en.isEmpty())
        {
            String[] sorted = new String[en.size()];
            int index = 0;
            for (int i = 0; i < en.size(); i++)
            {
                sorted[index++] = en.elementAt(i).toString();
            }

            for (int i1 = en.size(); i1 >= 0; i1--)
            {
                for (int i2 = 1; i2 < i1; i2++)
                {
                    if (sorted[i2 - 1].compareTo(sorted[i2]) > 0)
                    {
                        String tmp = sorted[i2 - 1];
                        sorted[i2 - 1] = sorted[i2];
                        sorted[i2] = tmp;
                    }
                }
            }

            en.removeAllElements();

            for (int i = 0; i < sorted.length; i++)
            {
                en.addElement(sorted[i]);
            }
        }

        return en;
    }

    /**
     * Appends a vector of string into the list with specific icon.
     * @param en Vector to be appended.
     * @param group Group icon.
     */
    private void appendVector(Vector en, Image group)
    {
        if (!en.isEmpty())
        {
            for (int i = 0; i < en.size(); i++)
            {
                this.append(en.elementAt(i).toString(), group);
            }
        }
    }

    /**
     * Gets index for string label.
     * @param label Label of the element.
     * @return Index of the element.
     */
    private int getIndex(String label)
    {
        int ret = -1;
        for (int i = 0; i < size(); i++)
        {
            if (label.equals(getString(i)))
            {
                ret = i;
            }
        }
        return ret;
    }
}
