package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.*;

import java.io.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;

/**
 * FileViewer makes a simple preview of a given file.
 * @author Adam Netocny
 */
public class FileViewer
        extends Form
        implements Displayable,
                   CommandListener
{

    /**
     * Maximal number of char shown in the text field.
     */
    private static final int MAX_FILE_SIZE = 2000;

    /**
     * Back command.
     */
    private final Command back = new Command("Back", Command.BACK, 0);

    /**
     * AlertScreen if something goes wrong.
     */
    private AlertScreen alert = null;

    /**
     * Constructor. Prepares the form for showing.
     * @param file File to be shown.
     * @throws IOException -----------
     */
    public FileViewer(String file) throws IOException
    {
        this(Connector.openInputStream("file://localhost/" + file), file);
    }

    /**
     *
     * @param in
     * @param file
     */
    public FileViewer(final InputStream in, final String file)
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME + " - " + file);

        append("Loading...");
        new Thread()
        {

            public void run()
            {
                super.run();

                byte[] b = new byte[512];
                byte[] out = new byte[0];
                try
                {
                    int i;
                    while ((i = in.read(b)) != -1)
                    {
                        byte[] old = out;
                        out = new byte[old.length + i];
                        System.arraycopy(old, 0, out, 0, old.length);
                        System.arraycopy(b, 0, out, old.length, i);
                    }

                    Image img = Image.createImage(out, 0, out.length);
                    in.close();

                    deleteAll();
                    append(img);
                }
                catch (Throwable ex)
                {
                    try
                    {
                        TextField t = new TextField(file, "",
                                FileViewer.MAX_FILE_SIZE, TextField.ANY);

                        t.setString(new String(out));
                        t.setConstraints(TextField.UNEDITABLE);
                        deleteAll();
                        append(t);
                    }
                    catch (Throwable ex1)
                    {
                        alert = new AlertScreen(file + " cannot be open! Message: "
                                                + ex.getMessage() + ", " + ex1.
                                getMessage(),
                                AlertType.INFO);
                        synchronized (TarArchivatorMidlet.lockMe)
                        {
                            TarArchivatorMidlet.me.updateScreen();
                        }
                    }
                    finally
                    {
                        try
                        {
                            if (in != null)
                                in.close();
                        }
                        catch (Throwable ex1)
                        {
                            alert = new AlertScreen(file + " cannot be open! Message: "
                                                    + ex.getMessage() + ", " + ex1.
                                    getMessage(),
                                    AlertType.INFO);
                            synchronized (TarArchivatorMidlet.lockMe)
                            {
                                TarArchivatorMidlet.me.updateScreen();
                            }
                        }
                    }
                }
            }
        }.start();

        addCommand(back);
        setCommandListener(this);
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        if (alert == null)
        {
            return this;
        }
        else
        {
            return alert;
        }
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == back)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.screenBack();
            }
        }
    }
}
