package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.format.processor.*;
import cz.cvut.fel.tar.archivator.transfer.*;

import java.util.*;
import javax.microedition.lcdui.*;

/**
 * Class for selecting and running of transfers.
 * @author Adam Netocny
 */
public class TransfersSelector
        extends List
        implements Displayable,
                   CommandListener
{

    /**
     * Select command.
     */
    private final Command select = new Command("Select", Command.OK, 1);

    /**
     * Back command.
     */
    private final Command back = new Command("Throw away", Command.BACK, 2);

    /**
     * Vector of loaded transfers.
     */
    private Vector transfers = null;

    /**
     * Result from the tar process.
     */
    private final Buffer in;

    /**
     * Constructor. Initializes default values.
     * @param in Result from the tar process.
     */
    public TransfersSelector(Buffer in)
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME
              + " - Select transfer channel...",
                IMPLICIT);
        System.gc();
        this.in = in;

        setSelectCommand(select);
        addCommand(back);

        setCommandListener(this);
    }

    /**
     * Gets the screen and reloads the list.
     * @return Screen.
     */
    public Screen getScreen()
    {
        this.deleteAll();

        processRegisteredTransfers();

        return this;
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == select)
        {
            if (transfers != null && !transfers.isEmpty())
            {
                if (this.getSelectedIndex() != -1)
                {
                    Transferable trans = (Transferable) transfers.elementAt(this.
                            getSelectedIndex());
                    trans.setBuffer(in);

                    synchronized (TarArchivatorMidlet.lockMe)
                    {
                        TarArchivatorMidlet.me.runTransfer(trans);
                    }
                }
            }
        }
        else if (c == back)
        {
            cleanUp();
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.showFileBrowser();
            }
        }
    }

    /**
     * Appends all available transfers to the list.
     */
    private void processRegisteredTransfers()
    {
        transfers = TransfersRegister.getTransfersWithState(
                Transferable.TRANSFER_AVAILABLE);

        int size = transfers.size();
        for (int i = 0; i < size; i++)
        {
            Transferable transfer = (Transferable) transfers.elementAt(i);
            this.append(transfer.getName(), transfer.getIcon());
        }
    }

    /**
     * Do clean-up.
     */
    private void cleanUp()
    {
        in.clearBuffer();
    }
}
