package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.format.TarEntry;
import cz.cvut.fel.tar.archivator.format.TarInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.lcdui.*;

/**
 * Displays content of a Tar file and allows file preview and untaring.
 * @author Adam Netocny
 */
public class UnTarScreen
        extends List
        implements Displayable,
                   CommandListener
{

    /**
     * Back command.
     */
    private final Command back = new Command("Back", Command.BACK, 2);

    /**
     * Back command.
     */
    private final Command untar = new Command("Untar...", Command.ITEM, 1);

    /**
     * Back command.
     */
    private final Command view = new Command("View", Command.ITEM, 0);

    /**
     * AlertScreen if something goes wrong.
     */
    private AlertScreen alert = null;

    /**
     * Tar file to be displayed and untared.
     */
    private final String file;

    /**
     * File image.
     */
    private Image file_image = null;

    /**
     * Folder image.
     */
    private Image folder_image = null;

    /**
     * Creates an instance of this class.
     * @param file Tar file to be displayed and untared.
     */
    public UnTarScreen(String file)
    {
        super("TAR preview - " + file.substring(file.lastIndexOf('/') + 1), List.IMPLICIT);
        this.file = file;

        try
        {
            folder_image = Image.createImage("/resources/folder.png");
        }
        catch (IOException ex)
        {
        }

        try
        {
            file_image = Image.createImage("/resources/file.png");
        }
        catch (IOException ex)
        {
        }

        loadContent();

        addCommand(back);
        addCommand(untar);
        setSelectCommand(view);
        setCommandListener(this);
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        if (alert == null)
        {
            return this;
        }
        else
        {
            return alert;
        }
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == back)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(TarArchivatorMidlet.me.
                        getFileBrowser(false).getScreen());
            }
        }
        else if (c == untar)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                String dest = file.substring(0, file.lastIndexOf('/') + 1);
                int i = 1;
                while (true)
                {
                    try
                    {
                        FileConnection conn = (FileConnection) Connector.open(
                                "file://localhost/" + dest + "tar_" + i + "/");
                        if (!conn.exists())
                        {
                            dest = dest + "tar_" + i + "/";
                            break;
                        }
                    }
                    catch (IOException ex)
                    {
                        alert = new AlertScreen(dest + "tar_" + i + " cannot be open! Message: "
                                                + ex.getMessage(),
                                AlertType.INFO);
                        TarArchivatorMidlet.me.switchDisplayable(getScreen());
                        return;
                    }
                    i++;
                }
                TarArchivatorMidlet.me.untar(file, dest);
            }
        }
        else if (c == view)
        {
            List curr = (List) d;
            final String currFile = curr.getString(curr.getSelectedIndex());

            InputStream in = getInputStreamForEntry(currFile);
            if (in == null)
            {
                synchronized (TarArchivatorMidlet.lockMe)
                {
                    TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                            "Directory cannot be displayed.", AlertType.INFO));
                }
                return;
            }

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(new FileViewer(in,
                        currFile).getScreen());
            }
        }
    }

    /**
     * Loads entries into this list.
     */
    private void loadContent()
    {
        try
        {
            TarInputStream in = new TarInputStream(Connector.openInputStream(
                    "file://localhost/" + file));
            TarEntry entry;
            while ((entry = in.getNextEntry()) != null)
            {
                if (entry.isDirectory())
                {
                    // this.append(entry.getName(), folder_image);
                }
                else
                {
                    this.append(entry.getName(), file_image);
                }
            }
            in.close();
        }
        catch (IOException ex)
        {
            alert = new AlertScreen(file + " cannot be open! Message: "
                                    + ex.getMessage(),
                    AlertType.INFO);
        }
    }

    /**
     * Returns stream for a file in the tar.
     * @param currFile Entry name.
     * @return Input stream for the entry.
     */
    private InputStream getInputStreamForEntry(String currFile)
    {
        try
        {
            TarInputStream in = new TarInputStream(Connector.openInputStream(
                    "file://localhost/" + file));
            TarEntry entry;
            while ((entry = in.getNextEntry()) != null)
            {
                if (entry.getName().equalsIgnoreCase(currFile))
                {
                    if (entry.isDirectory())
                    {
                        in.close();
                        return null;
                    }
                    else
                    {
                        return in;
                    }
                }
            }

            in.close();
            return null;
        }
        catch (IOException ex)
        {
            alert = new AlertScreen(file + " cannot be open! Message: "
                                    + ex.getMessage(),
                    AlertType.INFO);
            return null;
        }
    }
}
