package cz.cvut.fel.tar.archivator.processors;

/**
 * Default processor implementation.
 * @author Adam Netocny
 */
public abstract class Processor
        implements ProgressRunnable
{

    /**
     * Maximal progress value.
     */
    public static final int MAX_PROGRESS = 100;

    /**
     * Current progress.
     */
    protected int progress = 0;

    /**
     * Lock for thread-safe operations with progress value.
     */
    protected final Object lock = new Object();

    /**
     * Gets current progress value within rage <tt>zero</tt> and
     * <tt>getMaxProgress()</tt>.
     * @return Current progress.
     */
    public int getProgress()
    {
        synchronized (lock)
        {
            if (progress > MAX_PROGRESS)
                return MAX_PROGRESS;
            else if (progress < 0)
                return 0;
            else
                return progress;
        }
    }

    /**
     * Gets maximal progress value. Current progress can not be greater
     * then this value.
     * @return Maximal progress value.
     */
    public int getMaxProgress()
    {
        return MAX_PROGRESS;
    }

    /**
     * Gets label to be shown under progress bar.
     * @return String label.
     */
    public String getProgressLabel()
    {
        int val = new Double(
                (((double) getProgress()) / ((double) getMaxProgress())) * 100).
                intValue();
        if (val < 10)
            return "  " + val + "%";
        else if (val < 100)
            return " " + val + "%";
        else
            return val + "%";
    }
}
