package cz.cvut.fel.tar.archivator.processors;

/**
 * Runnable with progress.
 * @author Adam Netocny
 */
public interface ProgressRunnable
        extends Runnable
{

    /**
     * Gets current progress value within rage <tt>zero</tt> and
     * <tt>getMaxProgress()</tt>.
     * @return Current progress.
     */
    public int getProgress();

    /**
     * Gets maximal progress value. Current progress can not be greater
     * then this value.
     * @return Maximal progress value.
     */
    public int getMaxProgress();

    /**
     * Terminates this thread as soon as it is possible.
     */
    public void terminateAction();

    /**
     * Gets label to be shown under progress bar.
     * @return String label.
     */
    public String getProgressLabel();

    /**
     * Runs post process function to process the result.
     */
    public void runPostProcess();
}
