package cz.cvut.fel.tar.archivator.transfer;

import java.util.*;

/**
 * Register for all transfers.
 * @author Adam Netocny
 */
public class TransfersRegister
{

    /**
     * Vector of available transfers.
     */
    private static Vector transfers = new Vector();

    /**
     * Registers a transfer.
     * @param trans Transfer to be registered.
     */
    public static void registerTransfer(Transferable trans)
    {
        transfers.addElement(trans);
    }

    /**
     * Unregisters transfers with a specific class.
     * @param c Class of transfers to be removed.
     */
    public static void unregisterTransfer(Class c)
    {
        for (int i = 0; i < transfers.size(); i++)
        {
            Class c1 = transfers.elementAt(i).getClass();
            if (c1.equals(c))
            {
                transfers.removeElementAt(i);
            }
        }
    }

    /**
     * Gets transfers count.
     * @return Transfers count.
     */
    public static int getSize()
    {
        return transfers.size();
    }

    /**
     * Gets transfer on a specific index.
     * @param index Index of the transfer.
     * @return Transfer on the specific index.
     */
    public static Transferable getTransfer(int index)
    {
        return (Transferable) transfers.elementAt(index);
    }

    /**
     * Gets a vector of transfers with a specific state. If state is not
     * Transferable.TRANSFER_NOT_AVAILABLE then all transfers with state
     * Transferable.TRANSFER_NOT_AVAILABLE will be removed.
     * @param state Transfers state we're looking for.
     * @return Vector of transfers with a specific state.
     *
     * @see Transferable
     */
    public static Vector getTransfersWithState(int state)
    {
        Vector ret = new Vector();
        Vector rem = new Vector();
        for (int i = 0; i < transfers.size(); i++)
        {
            if (((Transferable) transfers.elementAt(i)).getState() == state)
                ret.addElement(transfers.elementAt(i));
            else if (((Transferable) transfers.elementAt(i)).getState() == Transferable.TRANSFER_NOT_AVAILABLE)
                rem.addElement(transfers.elementAt(i));
        }

        Enumeration e = rem.elements();
        while (e.hasMoreElements())
        {
            transfers.removeElement(e.nextElement());
        }

        return ret;
    }
}
