package cz.cvut.fel.tar.archivator.transfer.bluetooth;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import java.io.*;
import java.util.*;
import javax.bluetooth.*;
import javax.microedition.lcdui.*;

/**
 * Bluetooth device discovery. Searches for available devices.
 * @author Adam Netocny
 */
public class BluethootDeviceDiscovery
        implements DiscoveryListener
{

    /**
     * Available devices.
     */
    private final Vector remoteDevices = new Vector();

    /**
     * DiscoveryAgent.
     */
    private DiscoveryAgent discoveryAgent;

    /**
     * Selector for appending.
     */
    private final BluetoothSelector parent;

    /**
     * Constructor. Starts the search process.
     * @param parent Selectors screen.
     */
    public BluethootDeviceDiscovery(BluetoothSelector parent)
    {
        this.parent = parent;

        try
        {
            LocalDevice localDevice = LocalDevice.getLocalDevice();
            discoveryAgent = localDevice.getDiscoveryAgent();

            discoveryAgent.startInquiry(DiscoveryAgent.GIAC, this);
        }
        catch (Exception e)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                        "Bluetooth hardware is down.", AlertType.WARNING).
                        getScreen());
            }
        }
    }

    /**
     * This method is called each time when a remote Bluetooth device
     * has been discovered.
     * @param btDevice RemoteDevice
     * @param cod DeviceClass
     */
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod)
    {
        remoteDevices.addElement(btDevice);
        try
        {
            parent.append(btDevice.getFriendlyName(true));
        }
        catch (IOException ex)
        {
            parent.append(btDevice.getBluetoothAddress());
        }
    }

    /**
     * This method is called when the Bluetooth device discovery process
     * has ended.
     * @param discType int
     */
    public void inquiryCompleted(int discType)
    {
        if (discType == INQUIRY_COMPLETED)
        {
            parent.searchComplete();
        }
    }

    /**
     * This method is not used, but any implementation of DiscoveryListerer.java
     * must include these methods
     * @param transID int
     * @param servRecord ServiceRecord[]
     */
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord)
    {
    }

    /**
     * This method is not used, but any implementation of DiscoveryListerer.java
     * must include these methods
     * @param transID int
     * @param respCode 
     */
    public void serviceSearchCompleted(int transID, int respCode)
    {
    }

    /**
     * Gets a <tt>Vector</tt> of all available devices.
     * @return <tt>Vector</tt> of all available devices.
     */
    public Vector getRemoteDevices()
    {
        return remoteDevices;
    }
}
