package cz.cvut.fel.tar.archivator.transfer.bluetooth;

import javax.bluetooth.*;

/**
 * Bluetooth service discovery. Searches for available OBEX services for given
 * device.
 * @author Adam Netocny
 */
public class BluetoothServiceDiscovery
        extends Thread
        implements DiscoveryListener
{

    /**
     * OBEX UUID.
     */
    private static final UUID[] uuidSet =
    {
        new UUID(0x1105)
    };

    /**
     * Search attributes.
     */
    private static final int[] attrSet =
    {
        0x0100,
        0x0003,
        0x0004
    };

    /**
     * Device to be searched on.
     */
    private final RemoteDevice remoteDevice;

    /**
     * Selectors screen.
     */
    private final BluetoothSelector parent;

    /**
     * Constructor.
     * @param parent Selectors screen.
     * @param remoteDevice Device to be searched on.
     */
    public BluetoothServiceDiscovery(BluetoothSelector parent,
            RemoteDevice remoteDevice)
    {
        this.remoteDevice = remoteDevice;
        this.parent = parent;
    }

    /**
     * Starts the search process.
     */
    public void run()
    {
        try
        {
            LocalDevice localDevice = LocalDevice.getLocalDevice();
            DiscoveryAgent discoveryAgent = localDevice.getDiscoveryAgent();

            discoveryAgent.searchServices(attrSet, uuidSet, remoteDevice, this);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * This method is called when a new service is discovered on a remote
     * Bluetooth device.
     * @param transID int
     * @param servRecord ServiceRecord[]
     */
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord)
    {
        for (int i = 0; i < servRecord.length; i++)
        {
            parent.setService(servRecord[i].getConnectionURL(
                    ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false));
        }
    }

    /**
     * This method is called when the service search process is completed for
     * a remote Bluetooth device.
     * @param transID int
     * @param respCode int
     */
    public void serviceSearchCompleted(int transID, int respCode)
    {
        parent.searchComplete();
    }

    /**
     * This method is not used, but any implementation of DiscoveryListerer.java
     * must include these methods
     * @param discType
     */
    public void inquiryCompleted(int discType)
    {
    }

    /**
     * This method is not used, but any implementation of DiscoveryListerer.java
     * must include these methods
     * @param btDevice
     * @param cod
     */
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod)
    {
    }
}
