package cz.cvut.fel.tar.archivator.transfer.calculator;

import java.util.Date;

/**
 * Calculates remaining time and transfer speed.
 * @author Adam Netocny
 */
public class TransferSpeedCalculator
{

    /**
     * Bytes to be transfered.
     */
    private long totalBytes;

    /**
     * Bytes transfered.
     */
    private long transferedBytes;

    /**
     * Time per byte.
     */
    private double time;

    /**
     * Last timestamp.
     */
    private Date timestamp;

    /**
     * Constructor.
     * @param totalBytes Total bytes to be transfered.
     */
    public TransferSpeedCalculator(long totalBytes)
    {
        this.totalBytes = totalBytes;
        this.transferedBytes = 0L;
        this.time = 0;
        timestamp = new Date();
    }

    /**
     * Calculates remaining time.
     * @param bytesTransferedSinceLastCall Bytes transfered since last call.
     * @return String in format HHh MMm SSs
     */
    public String calculateRemainingTime(long bytesTransferedSinceLastCall)
    {
        long last = timestamp.getTime();
        timestamp = new Date();
        long newts = timestamp.getTime();
        long dur = newts - last;
        double newTime = ((double) dur) / ((double) bytesTransferedSinceLastCall);

        if (time == 0)
        {
            time = newTime;
        }
        else
        {
            //TODO: opravit priemer
            if (newTime != 0)
            {
                time = Math.sqrt((time) * (newTime));
            }
        }

        String res = "";

        transferedBytes += bytesTransferedSinceLastCall;

        long timeRemaining = (long) ((totalBytes - transferedBytes) * time);

        long hoursD = timeRemaining / 3600000L;
        timeRemaining = timeRemaining - (hoursD * 3600000L);
        long minD = timeRemaining / 60000L;
        timeRemaining = timeRemaining - (minD * 60000L);
        long secD = timeRemaining / 1000L;

        String seconds = Integer.toString((int) secD);
        String minutes = Integer.toString((int) minD);
        String hours = Integer.toString((int) hoursD);
        if (seconds.length() < 2)
        {
            seconds = "0" + seconds;
        }
        if (minutes.length() < 2)
        {
            minutes = "0" + minutes;
        }
        if (hours.length() < 2)
        {
            hours = "0" + hours;
        }

        res = seconds + "s " + res;

        res = minutes + "m " + res;

        res = hours + "h " + res;

        return "Time remaining: " + res;
    }
}
