package cz.cvut.fel.tar.archivator.transfer.filesystem;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import cz.cvut.fel.tar.archivator.transfer.*;
import cz.cvut.fel.tar.archivator.format.processor.*;
import cz.cvut.fel.tar.archivator.transfer.calculator.TransferSpeedCalculator;

import java.io.*;
import java.util.Enumeration;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

/**
 * For testing purposes only.
 * @author Adam Netocny
 */
public class FileSystemProcessor
        extends TransferProcessor
{

    /**
     * Termination variable.
     */
    private boolean run = true;

    /**
     * Lock for run variable.
     */
    private final Object lockRun = new Object();

    /**
     * Number of bytes written to the file.
     */
    private long written = 0;

    /**
     * Output of calculator.
     */
    private String calculatorOutput = "";

    /**
     * Lock for calculator output.
     */
    private final Object calculatorOutputLock = new Object();

    /**
     * Terminates this action as soon as possible.
     */
    public void terminateAction()
    {
        synchronized (lockRun)
        {
            run = false;
        }
    }

    /**
     * Run function. Writes the content of the buffer to a file.
     */
    public void run()
    {
        try
        {
            Enumeration e = FileSystemRegistry.listRoots();
            String root = "";
            while (e.hasMoreElements())
            {
                root = (String) e.nextElement();
            }

            FileConnection conn = (FileConnection) Connector.open(
                    "file://localhost/" + root + "tar_archivator.tar");

            aditionalInfo = "Archive stored as: " + root + "tar_archivator.tar";

            if (!conn.exists())
            {
                conn.create();
            }

            long size = in.getTotalSize();

            TransferSpeedCalculator transferSpeedCalculator = new TransferSpeedCalculator(
                    size);

            BufferedInputStream bufin = new BufferedInputStream(in);

            in = null;

            OutputStream out = conn.openOutputStream();

            byte[] b = new byte[255];
            int c = 0;
            while ((c = bufin.read(b)) != -1)
            {
                out.write(b, 0, c);

                synchronized (calculatorOutputLock)
                {
                    calculatorOutput = transferSpeedCalculator.
                            calculateRemainingTime(c);
                }

                synchronized (lock)
                {
                    written += c;
                    progress = new Double(
                            ((double) written) / ((double) size) * 100).intValue();
                }
                synchronized (lockRun)
                {
                    if (!run)
                    {
                        bufin.getBuffer().clearBuffer();
                        out.flush();
                        out.close();
                        conn.delete();
                        conn.close();
                        bufin.close();

                        Thread.sleep(1000);

                        TransfersRegister.unregisterTransfer(
                                FileSystemProcessor.class);

                        new FileSystemProcessor();
                        return;
                    }
                }
            }

            out.flush();
            out.close();
            bufin.close();
            conn.close();

            Thread.sleep(1000);

            TransfersRegister.unregisterTransfer(FileSystemProcessor.class);
            new FileSystemProcessor();
        }
        catch (Throwable ex)
        {
            TransfersRegister.unregisterTransfer(FileSystemProcessor.class);
            new FileSystemProcessor();
            in.clearBuffer();

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                        ex.getMessage(), AlertType.ERROR).getScreen());
            }
        }
    }

    /**
     * Gets the name of this transfer.
     * @return Name of this transfer.
     */
    public String getName()
    {
        return "FileSystem";
    }

    /**
     * Gets the icon for this transfer type.
     * @return Icon for this transfer type.
     */
    public Image getIcon()
    {
        Image folder = null;
        try
        {
            folder = Image.createImage("/resources/folder.png");
        }
        catch (IOException ex)
        {
        }

        return folder;
    }

    /**
     * Gets label to be shown under progress bar.
     * @return String label.
     */
    public String getProgressLabel()
    {
        synchronized (calculatorOutputLock)
        {
            return super.getProgressLabel() + "\n" + calculatorOutput;
        }
    }
}
